package nl.voidcorp.dbot.music

import com.sedmelluq.discord.lavaplayer.player.AudioPlayer
import com.sedmelluq.discord.lavaplayer.player.event.AudioEventAdapter
import com.sedmelluq.discord.lavaplayer.source.http.HttpAudioTrack
import com.sedmelluq.discord.lavaplayer.source.soundcloud.SoundCloudAudioSourceManager
import com.sedmelluq.discord.lavaplayer.source.soundcloud.SoundCloudAudioTrack
import com.sedmelluq.discord.lavaplayer.source.youtube.YoutubeAudioTrack
import com.sedmelluq.discord.lavaplayer.track.AudioTrack
import com.sedmelluq.discord.lavaplayer.track.AudioTrackEndReason
import net.dv8tion.jda.core.EmbedBuilder
import net.dv8tion.jda.core.entities.*
import nl.voidcorp.dbot.log
import java.awt.Color
import java.time.LocalDateTime
import java.util.*
import kotlin.concurrent.timerTask


class TrackScheduler(val player: AudioPlayer, val guild: Guild, channel: VoiceChannel) : AudioEventAdapter() {
    val musicChannel: TextChannel

    init {
        player.addListener(this)

        val audio = guild.audioManager

        audio.sendingHandler = AudioPlayerSendHandler(player)
        audio.openAudioConnection(channel)

        musicChannel = guild.getTextChannelsByName("music-bot", true).firstOrNull()
                ?: (guild.getTextChannelsByName("music", true).firstOrNull()
                ?: (guild.getTextChannelsByName("bot", true).firstOrNull()
                        ?: (guild.getTextChannelsByName("general", true).firstOrNull()
                                ?: guild.defaultChannel!!)))

    }

    private val q = ArrayDeque<AudioTrack>()
    /*override fun onEvent(event: AudioEvent) {
        if (event is TrackEndEvent) {

        }
    }*/

    override fun onTrackEnd(player: AudioPlayer, track: AudioTrack, endReason: AudioTrackEndReason) {
        if (q.isNotEmpty()) {
            player.playTrack(q.poll())
        } else {
            stopPlay(endReason)
            player.destroy()
        }
    }

    override fun onTrackStart(player: AudioPlayer, track: AudioTrack) {
        musicChannel.sendMessage(getCurrentTrackInfo()).append("Now playing").queue()

        /*if (track is YoutubeAudioTrack) {
            if (track.userData is Member)
        } else if (track is SoundCloudAudioTrack) {
            val scsm = track.sourceManager as SoundCloudAudioSourceManager
            scsm.updateClientId()
            val art = khttp.get("http://api.soundcloud.com/tracks/${track.info.identifier}?client_id=${scsm.clientId}", headers = mapOf("Content-Type" to "application/json")).jsonObject["artwork_url"].toString().replace("large", "t300x300")

            if (track.userData is Member)
                musicChannel.sendMessage(EmbedBuilder()
                        .setFooter("Requested by ${(track.userData as Member).effectiveName}", (track.userData as Member)
                                .user.effectiveAvatarUrl).setAuthor(track.info.author).setTitle(track.info.title, track.info.uri)
                        .setThumbnail(art)
                        .setTimestamp(LocalDateTime.now()).setColor(Color.decode("#ff8800")).build()).append("Now playing").queue()


        }
*/

    }

    fun queue(track: AudioTrack, member: Member) {
        track.userData = member
        if (q.isEmpty() && player.playingTrack == null) {
            log.info("Queue is empty, playing a track")
            player.playTrack(track)
        } else {
            log.info("Track Queue'd")
            q.offer(track)
        }
    }

    fun insertFront(track: AudioTrack, member: Member) {
        track.userData = member
        if (q.isEmpty() && player.playingTrack == null) {
            log.info("Queue is empty, playing a track")
            player.playTrack(track)
        } else {
            log.info("Track inserted")
            q.push(track)
        }
    }


    fun skip() {
        /*if (q.isNotEmpty()) {
            log.info("skipped Track!")
            player.playTrack(q.remove())
        } else {
            stopPlay(AudioTrackEndReason.REPLACED)
            player.destroy()
        }*/
        player.stopTrack()
    }

    fun isQueueEmpty() = q.isEmpty()

    fun isSongPlaying() = player.playingTrack != null

    fun stopPlay(endReason: AudioTrackEndReason) {

        if (endReason != AudioTrackEndReason.REPLACED) {
            guildMusicMap.remove(guild.idLong)
            musicChannel.sendMessage(EmbedBuilder().setColor(musicChannel.guild.selfMember.color).setTitle("I'm done here").setDescription("There are no more songs left in the queue.").setTimestamp(LocalDateTime.now()).build()).queue()

            Timer().schedule(timerTask {
                guild.audioManager.closeAudioConnection()
            }, 2000)

        }
    }

    fun getCurrentTrackInfo(): MessageEmbed = getTrackInfo(player.playingTrack)

    fun getTrackInfo(track: AudioTrack): MessageEmbed {
        if (track is YoutubeAudioTrack) {
            if (track.userData is Member)
                return EmbedBuilder()
                        .setFooter("Requested by ${(track.userData as Member).effectiveName}", (track.userData as Member)
                                .user.effectiveAvatarUrl).setAuthor(track.info.author).setTitle(track.info.title, track.info.uri)
                        .setThumbnail("https://img.youtube.com/vi/${track.info.identifier}/hqdefault.jpg")
                        .setTimestamp(LocalDateTime.now()).setColor(Color.decode("#ff0000")).build()
        } else if (track is SoundCloudAudioTrack) {
            val scsm = track.sourceManager as SoundCloudAudioSourceManager
            scsm.updateClientId()
            val art = khttp.get("http://api.soundcloud.com/tracks/${track.info.identifier}?client_id=${scsm.clientId}", headers = mapOf("Content-Type" to "application/json")).jsonObject["artwork_url"].toString().replace("large", "t300x300")

            if (track.userData is Member)
                return EmbedBuilder()
                        .setFooter("Requested by ${(track.userData as Member).effectiveName}", (track.userData as Member)
                                .user.effectiveAvatarUrl).setAuthor(track.info.author).setTitle(track.info.title, track.info.uri)
                        .setThumbnail(if (art == "null") null else art)
                        .setTimestamp(LocalDateTime.now()).setColor(Color.decode("#ff5500")).build()

        } else if (track is HttpAudioTrack) {
            if (track.userData is Member)
                return EmbedBuilder()
                        .setFooter("Requested by ${(track.userData as Member).effectiveName}", (track.userData as Member)
                                .user.effectiveAvatarUrl).setAuthor(track.info.author).setTitle(track.info.title, track.info.uri)
                        .setTimestamp(LocalDateTime.now()).setColor(Color.decode("#005A9C")).build()
        }

        return EmbedBuilder()
                .setFooter("Requested by ${(track.userData as Member).effectiveName}", (track.userData as Member)
                        .user.effectiveAvatarUrl).setAuthor(track.info.author).setTitle(track.info.title, track.info.uri)
                .setTimestamp(LocalDateTime.now()).setColor(musicChannel.guild.selfMember.color).build()
    }

    fun getTrackList(member: Member): MessageEmbed {
        return EmbedBuilder().setTitle("Hey ${member.effectiveName}, here is the playlist").setTimestamp(LocalDateTime.now()).setColor(musicChannel.guild.selfMember.color)
                .setFooter("Requested by ${member.effectiveName}", member.user.effectiveAvatarUrl).setDescription(
                        "**${player.playingTrack.info.title}**, requested by ${if (player.playingTrack.userData is Member) (player.playingTrack.userData as Member).effectiveName else "someone unknown..."} (*now playing*)\n" +
                                q.joinToString(separator = "\n") { "**${it.info.title}**, requested by ${if (it.userData is Member) (it.userData as Member).effectiveName else "someone unknown..."}" }
                ).build()
    }

}

